
// xiAPI header file

#ifndef __XIAPI_H
#define __XIAPI_H

#ifdef WIN32
#else
// linux
#include "wintypedefs.h"
#endif

#ifdef XIAPI_EXPORTS
#define XIAPI __declspec(dllexport)
#else
#define XIAPI __declspec(dllimport)
#endif

typedef int XI_RETURN;

#ifdef __cplusplus
extern "C" {
#endif

/** @name Error codes
         Definitions of the error codes used in API

         @note Most functions return XI_OK on success, an error code otherwise
   */
   /** @{ */
#define XI_OK 							0		//!< Function call succeeded
	//!< Errors code 1-100 see m3api.h file
#define XI_UNKNOWN_PARAM				100		//!< Unknown parameter
#define XI_WRONG_PARAM_VALUE			101		//!< Wrong parameter value
#define XI_WRONG_PARAM_TYPE				103		//!< Wrong parameter type
#define XI_WRONG_PARAM_SIZE				104		//!< Wrong parameter size
#define XI_BUFFER_TOO_SMALL				105		//!< Input buffer too small
#define XI_NOT_SUPPORTED_PARAM			106		//!< Parameter info not supported
#define XI_NOT_SUPPORTED_PARAM_INFO		107		//!< Parameter info not supported
#define XI_NOT_SUPPORTED_DATA_FORMAT	108		//!< Data format not supported
#define XI_READ_ONLY_PARAM				109		//!< Read only parameter
#define XI_NO_DEVICES_FOUND				110		//!< No devices found

/*************************************************************************************/
/**
 *\enum XI_IMG_FORMAT
 * \brief structure containing information about output image format
 */
typedef enum
{
	XI_MONO8 = 0,									//!< 8 bits per pixel 
	XI_MONO16 = 1,									//!< 16 bits per pixel
	XI_RGB24 = 2,	//unsupported yet				//!< RGB data format
	XI_RGB32 = 3									//!< RGBA data format	
}XI_IMG_FORMAT;		

/*************************************************************************************/
/**
 *\enum XI_BP
 * \brief structure containing information about buffer policy(can be safe, data will be copied to user/app buffer or unsafe, user will get internally allocated buffer without data copy).
 */
typedef enum
{
	XI_BP_UNSAFE = 0,								//!< User gets pointer to internally allocated circle buffer and data may be overwritten by device.
	XI_BP_SAFE = 1									//!< Data from device will be copied to user allocated buffer or xiApi allocated memory.
}XI_BP;

/*************************************************************************************/
//
/**
 *\struct XI_IMG
 * \brief structure containing information about incoming image.
 */
typedef struct
{
	DWORD size;									//!< size of structure.
	LPVOID bp;									//!< pointer to data. If NULL, xiApi allocates new buffer.
	DWORD bp_size;								//!< buffer size.
	XI_IMG_FORMAT frm;							//!< format of incoming data.	
	DWORD width;								//!< width of incoming image.
	DWORD height;								//!< height of incoming image.
	DWORD nframe;								//!< frame number(reset by exposure, gain, downsampling change). 
	//V1
	DWORD tsSec;								//!< TimeStamp in seconds
	DWORD tsUSec;								//!< TimeStamp in microseconds
	DWORD GPI_level;							//!< Input level
	//V2
}XI_IMG, *LPXI_IMG;

#define SIZE_XI_IMG_V1	28
#define SIZE_XI_IMG_V2	40

/*************************************************************************************/
//
/**
 *\enum XI_TRG_SOURCE
 * \brief structure containing information about trigger functionality
 */
typedef enum
{
	XI_TRG_OFF = 0,								//!< Camera works in free run mode.
	XI_TRG_EDGE_RISING = 1,						//!< External trigger (rising edge).
	XI_TRG_EDGE_FALLING = 2,					//!< External trigger (falling edge).	
	XI_TRG_SOFTWARE = 3							//!< Software(manual) trigger.
}XI_TRG_SOURCE;

/*************************************************************************************/
//
/**
 *\enum XI_GPI_MODE
 * \brief structure containing information about GPI functionality
 */
typedef enum
{
	XI_GPI_OFF = 0,								//!< Input off.
	XI_GPI_TRIGGER = 1,							//!< Trigger input
	XI_GPI_EXT_EVENT = 2						//!< External signal input
}XI_GPI_MODE;

/*************************************************************************************/
//
/**
 *\enum XI_GPO_MODE
 * \brief structure containing information about GPO functionality
 */
typedef enum
{
	XI_GPO_OFF = 0,								//!< Output off
	XI_GPO_ON = 1,								//!< Logical level.	
	XI_GPO_FRAME_ACTIVE = 2,					//!< On during exposure(integration) time + readout time + data transfer time
	XI_GPO_FRAME_ACTIVE_NEG = 3,				//!< Off during exposure(integration) time + readout time + data transfer time
	XI_GPO_EXPOSURE_ACTIVE = 4,					//!< On during exposure(integration) time
	XI_GPO_EXPOSURE_ACTIVE_NEG = 5				//!< Off during exposure(integration) time
}XI_GPO_MODE;

/*************************************************************************************/
//
/**
 *\enum XI_LED_MODE
 * \brief structure containing information about LED functionality
 */
typedef enum {
	XI_LED_HEARTBEAT				= 0x000,	//!< set led to blink if link is ok	(led 1), heartbeat (led 2)
	XI_LED_TRIGGER_ACTIVE			= 0x001,	//!< set led to blink if trigger detected
	XI_LED_EXT_EVENT_ACTIVE		= 0x002,		//!< set led to blink if external signal detected
	XI_LED_ACQUISITION				= 0x003,	//!< set led to blink if data streaming
	XI_LED_EXPOSURE_ACTIVE			= 0x004,	//!< set led to blink if sensor integration time
	XI_LED_FRAME_ACTIVE			= 0x005,		//!< set led to blink if device busy/not busy
	XI_LED_LINK					= 0x006,		//!< set led to blink if link is ok
	XI_LED_OFF						= 0x007,	//!< set led to zero
	XI_LED_ON						= 0x008,	//!< set led to one
	// etc
} XI_LED_MODE;

// Switch defines
#define XI_ON 1
#define XI_OFF 0

/*************************************************************************************/
//Device parameters
/*************************************************************************************/
/**
 *\enum PRM_TYPE
 * \brief structure containing information about parameters type
 */
typedef enum
{
	xiTypeInteger = 0,
	xiTypeFloat = 1,
	xiTypeString = 2 
}XI_PRM_TYPE;

//!< Device info parameters
#define XI_PRM_DEVICE_NAME	"device_name"		//!< Return device name
												//!< xiGetParam(handle, PRM_DEVICE_NAME PRM_INFO or "device_name:info", char *, size, TypeString);

#define XI_PRM_DEVICE_TYPE "device_type"		//!< Return device type
												//!< xiGetParam(handle, PRM_DEVICE_TYPE PRM_INFO or "device_type:info", char *, size, TypeString); 

#define XI_PRM_DEVICE_SN "device_sn"			//!< Return device serian number
												//!< xiGetParam(handle, PRM_DEVICE_SN PRM_INFO or "device_sn:info", &int, sizeof(int), TypeInteger);
//!< Device acquisition settings
#define XI_PRM_EXPOSURE "exposure"				//!< Exposure time in microseconds, 
												//!< xiSetParam(handle, PRM_EXPOSURE or "exposure", &int, sizeof(int), TypeInteger); 
												//!< By default 0 

#define XI_PRM_GAIN "gain"						//!< Gain in dB,  
												//!< xiSetParam(handle, PRM_GAIN or "gain", &float, sizeof(float), TypeFloat);
												//!< By default 0

#define XI_PRM_DOWNSAMPLING "downsampling"		//!< Change image resolution by binning or skipping.
												//!< xiSetParam(handle, PRM_DOWNSAMPLING or "downsampling", &int, sizeof(int), TypeInteger); 
												//!< By default 1

#define XI_PRM_IMAGE_DATA_FORMAT "imgdataformat"//!< Output data format.
												//!< xiSetParam(handle, PRM_IMAGE_DATA_FORMAT or "imgdataformat", &int, sizeof(int), TypeInteger); 
												//!< By default MONO8

#define XI_PRM_FRAMERATE "framerate"			//!< Define framerate
												//!< xiSetParam(handle, PRM_FRAMERATE or "framerate", &float, sizeof(float), TypeFloat); 
												//!< By default (depend on sensor, downsalmping and resolution)

#define XI_PRM_BUFFER_POLICY "buffer_policy"	//!< Data move policy
												//!< xiSetParam(handle, PRM_BUFFER_POLICY or "buffer_policy", &int, sizeof(int), TypeInteger); 
												//!< By default BP_UNSAFE

#define XI_PRM_OFFSET_X "offsetX"				//!< Horizontal offset from the origin to the area of interest (in pixels).
												//!< xiSetParam(handle, PRM_OFFSET_X or "offsetX", &int, sizeof(int), TypeInteger); 
												//!< By default 0
												// NOTE:
												//!< Always update image width before setting offset_X for the equation
												//!< offsetX + width <= full_image_width to be true otherwise XI_WRONG_PARAM_VALUE will be returned

#define XI_PRM_OFFSET_Y "offsetY"				//!< Vertical offset from the origin to the area of interest (in pixels).
												//!< xiSetParam(handle, PRM_OFFSET_Y or "offsetY", &int, sizeof(int), TypeInteger); 
												//!< By default 0
												// NOTE:
												//!< Always update image height before setting offset_Y for the equation
												//!< offsetY + height <= full_image_height to be true otherwise XI_WRONG_PARAM_VALUE will be returned

#define XI_PRM_WIDTH "width"					//!< Width of the Image provided by the device (in pixels).
												//!< xiSetParam(handle, PRM_WIDTH or "width", &int, sizeof(int), TypeInteger); 
												//!< By default (depend on sensor, downsalmping and offsetX)

#define XI_PRM_HEIGHT "height"					//!< Height of the Image provided by the device (in pixels).
												//!< xiSetParam(handle, PRM_HEIGHT or "height", &int, sizeof(int), TypeInteger); 
												//!< By default (depend on sensor, downsalmping and offsetY)

#define XI_PRM_LUT_EN "LUTEnable"				//!< Activates LUT.
												//!< xiSetParam(handle, PRM_LUT_EN or "LUTEnable", &iny, sizeof(int), TypeInteger); 
												//!< By default 0(disabled)

#define XI_PRM_LUT_INDEX "LUTIndex"				//!< Control the index (offset) of the coefficient to access in the LUT.
												//!< xiSetParam(handle, PRM_LUT_INDEX or "LUTIndex", &int, sizeof(int), TypeInteger); 
												//!< By default 0

#define XI_PRM_LUT_VALUE "LUTValue"				//!< Value at entry LUTIndex of the LUT
												//!< xiSetParam(handle, PRM_LUT_VALUE or "LUTValue", &int, sizeof(int), TypeInteger); 
												//!< By default 0

#define XI_PRM_TRG_SOURCE "trigger_source"		//!< Defines source of trigger.
												//!< xiSetParam(handle, PRM_TRG_SOURCE or "trigger_source", &int, sizeof(int), TypeInteger); 
												//!< By default TRG_OFF

#define XI_PRM_TRG_SOFTWARE "trigger_software"	//!< Generates an internal trigger. PRM_TRG_SOURCE must be set to TRG_SOFTWARE.
												//!< xiSetParam(handle, PRM_TRG_SOFTWARE or "trigger_software", NULL, 0, NULL); 
												//!< By default -
//!< Device IO settings
#define XI_PRM_GPI_SELECTOR	"gpi_selector"		//!< Selects GPI 
												//!< xiSetParam(handle, PRM_GPI_SELECTOR or "gpi_selector", &int, sizeof(int), TypeInteger); 
												//!< By default 1

#define XI_PRM_GPI_MODE	"gpi_mode"				//!< Defines GPI functionality
												//!< xiSetParam(handle, PRM_GPI_MODE or "gpi_mode", &int, sizeof(int), TypeInteger); 
												//!< By default 0

#define XI_PRM_GPI_LEVEL	"gpi_level"			//!< GPI level
												//!< xiGetParam(handle, XI_PRM_GPI_LEVEL or "gpi_level", &int, sizeof(int), TypeInteger); 
												//!< By default -

#define XI_PRM_GPO_SELECTOR	"gpo_selector"		//!< Selects GPO 
												//!< xiSetParam(handle, PRM_GPO_SELECTOR or "gpo_selector", &int, sizeof(int), TypeInteger); 
												//!< By default 1

#define XI_PRM_GPO_MODE	"gpo_mode"				//!< Defines GPO functionality
												//!< xiSetParam(handle, PRM_GPO_MODE or "gpo_mode", &int, sizeof(int), TypeInteger); 
												//!< By default 0

#define XI_PRM_LED_SELECTOR	"led_selector"		//!< Selects LED 
												//!< xiSetParam(handle, PRM_LED_SELECTOR or "led_selector", &int, sizeof(int), TypeInteger); 
												//!< By default 1

#define XI_PRM_LED_MODE	"led_mode"				//!< Defines LED functionality
												//!< xiSetParam(handle, PRM_LED_MODE or "led_mode", &int, sizeof(int), TypeInteger); 
												//!< By default 0
//!< Device specific parameters

//!< Parameters info 
#define XI_PRM_INFO_MIN ":min"					//!< Parameter minimum
#define XI_PRM_INFO_MAX ":max"					//!< Parameter maximum	
#define XI_PRM_INFO ":info"						//!< Parameter info	

/*************************************************************************************/
/**
   \brief Return number of discovered devices
   
   Returns the pointer to the number of all discovered devices.

   @param[out] pNumberDevices			number of discovered devices
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiGetNumberDevices(OUT PDWORD pNumberDevices);
/**
   \brief Initialize device
   
   This function prepares the camera's software for work.
   It populates structures, runs initializing procedures, allocates resources - prepares the camera for work.

	\note Function creates and returns handle of the specified device. To de-initialize the camera and destroy the handler xiCloseDevice should be called.	

   @param[in] DevId						index of the device
   @param[out] hDevice					handle to device
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiOpenDevice(IN DWORD DevId, OUT PHANDLE hDevice);
/**
   \brief Uninitialize device
   
   Closes camera handle and releases allocated resources.

   @param[in] hDevice					handle to device
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiCloseDevice(IN HANDLE hDevice);
/**
   \brief Start image acquisition
   
   Begins the work cycle and starts data acquisition from the camera.

   @param[in] hDevice					handle to device
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiStartAcquisition(IN HANDLE hDevice);
/**
   \brief Stop image acquisition
   
   Ends the work cycle of the camera and stops data acquisition.

   @param[in] hDevice					handle to device
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiStopAcquisition(IN HANDLE hDevice);
/**
   \brief Return pointer to image structure
   
   Allows the user to retrieve the frame into LPXI_IMG structure.

   @param[in] hDevice					handle to device
   @param[in] timeout					time interval required to wait for the image (in milliseconds).
   @param[out] img						pointer to image info structure
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiGetImage(IN HANDLE hDevice, IN DWORD timeout, OUT LPXI_IMG img);
/**
   \brief Set device parameter
   
   Allows the user to control device.

   @param[in] hDevice					handle to device
   @param[in] prm						parameter name string.
   @param[in] val						pointer to parameter set value.
   @param[in] size						size of val.
   @param[in] type						val data type.
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiSetParam(IN HANDLE hDevice, const char* prm, void* val, DWORD size, XI_PRM_TYPE type);
/**
   \brief Get device parameter
   
   Allows the user to get the current device state and information.

   @param[in] hDevice					handle to device
   @param[in] prm						parameter name string.
   @param[in] val						pointer to parameter set value.
   @param[in] size						pointer to integer.
   @param[in] type						pointer to type container.
   @return XI_OK on success, error value otherwise.
 */
XIAPI XI_RETURN __cdecl xiGetParam(IN HANDLE hDevice, const char* prm, void* val, DWORD * size, XI_PRM_TYPE * type);

#ifdef __cplusplus
}
#endif

#endif /* __XIAPI_H */